# -*- coding: utf-8 -*-
from six import text_type, binary_type
from prettytable import PrettyTable
from os import path
from contextlib import closing
from xml.dom import minidom
from six import text_type
from hashlib import md5, sha1
from tqdm import tqdm
import time
import requests
import logging
import sys
import os
import base64
import datetime
import pytz
import yaml
import fnmatch
import copy

maplist = {
            'x-cos-copy-source-If-Modified-Since': 'CopySourceIfModifiedSince',
            'Content-Length': 'ContentLength',
            'x-cos-server-side-encryption-cos-kms-key-id': 'SSEKMSKeyId',
            'x-cos-server-side-encryption-customer-algorithm': 'SSECustomerAlgorithm',
            'If-Unmodified-Since': 'IfUnmodifiedSince',
            'response-content-language': 'ResponseContentLanguage',
            'Metadata': 'Metadata',
            'x-cos-grant-read': 'GrantRead',
            'x-cos-copy-source-If-None-Match': 'CopySourceIfNoneMatch',
            'Content-Language': 'ContentLanguage',
            'x-cos-server-side-encryption': 'ServerSideEncryption',
            'response-expires': 'ResponseExpires',
            'Expires': 'Expires',
            'Content-MD5': 'ContentMD5',
            'response-content-disposition': 'ResponseContentDisposition',
            'Referer': 'Referer',
            'x-cos-grant-full-control': 'GrantFullControl',
            'response-content-encoding': 'ResponseContentEncoding',
            'Content-Disposition': 'ContentDisposition',
            'If-Modified-Since': 'IfModifiedSince',
            'versionId': 'VersionId',
            'response-content-type': 'ResponseContentType',
            'Range': 'Range',
            'x-cos-server-side-encryption-customer-key-MD5': 'SSECustomerKeyMD5',
            'x-cos-acl': 'ACL',
            'x-cos-copy-source-If-Match': 'CopySourceIfMatch',
            'Content-Encoding': 'ContentEncoding',
            'x-cos-copy-source-If-Unmodified-Since': 'CopySourceIfUnmodifiedSince',
            'response-cache-control': 'ResponseCacheControl',
            'x-cos-server-side-encryption-customer-key': 'SSECustomerKey',
            'x-cos-grant-write': 'GrantWrite',
            'If-Match': 'IfMatch',
            'x-cos-storage-class': 'StorageClass',
            'Cache-Control': 'CacheControl',
            'If-None-Match': 'IfNoneMatch',
            'Content-Type': 'ContentType',
            'Pic-Operations': 'PicOperations',
            'x-cos-traffic-limit': 'TrafficLimit',
        }


def mapped(headers):
    """coscmd到pythonsdk参数的一个映射"""
    _headers = dict()
    _meta = dict()
    for i in headers:
        if i in maplist:
            _headers[maplist[i]] = headers[i]
        elif i.startswith('x-cos-meta-'):
            _meta[i] = headers[i]
        else:
            raise Exception('No Parameter Named ' + i + ' Please Check It')
    if len(_meta) > 0:
        _headers['Metadata'] = _meta
    return _headers


def to_bytes(s):
    """将字符串转为bytes"""
    if isinstance(s, text_type):
        try:
            return s.encode('utf-8')
        except UnicodeEncodeError as e:
            raise Exception('your unicode strings can not encoded in utf8, utf8 support only!')
    return s


def to_unicode(s):
    """将字符串转为unicode"""
    if isinstance(s, binary_type):
        try:
            return s.decode('utf-8')
        except UnicodeDecodeError as e:
            raise Exception('your bytes strings can not be decoded in utf8, utf8 support only!')
    return s


def get_file_md5(local_path):
    """获取文件md5"""
    md5_value = md5()
    with open(local_path, "rb") as f:
        while True:
            data = f.read(2048)
            if not data:
                break
            md5_value.update(data)
    return md5_value.hexdigest()


def gen_local_file(filename, filesize):
    with open(filename, 'wb') as f:
        f.write(os.urandom(filesize * 1024 * 1024))
    return 0


def to_printable_str(s):
    if isinstance(s, text_type):
        return s.encode('utf-8')
    else:
        return s


def getTagText(root, tag):
    node = root.getElementsByTagName(tag)[0]
    rc = ""
    for node in node.childNodes:
        if node.nodeType in (node.TEXT_NODE, node.CDATA_SECTION_NODE):
            rc = rc + node.data


def get_md5_filename(local_path, cos_path):
    ori_file = os.path.abspath(os.path.dirname(
        local_path)) + "!!!" + str(os.path.getsize(local_path)) + "!!!" + cos_path
    m = md5()
    m.update(to_bytes(ori_file))
    return os.path.expanduser('~/.tmp/' + m.hexdigest())


def query_yes_no(question, default="no"):
    valid = {"yes": True, "y": True, "ye": True,
             "no": False, "n": False}
    if default is None:
        prompt = " [y/n] "
    elif default == "yes":
        prompt = " [Y/n] "
    elif default == "no":
        prompt = " [y/N] "
    else:
        raise ValueError("invalid default answer: '%s'" % default)
    while True:
        sys.stdout.write(question + prompt)
        sys.stdout.flush()
        if sys.version > '3':
            choice = input()
        else:
            choice = raw_input()
        if default is not None and choice == '':
            return valid[default]
        elif choice in valid:
            return valid[choice]
        else:
            sys.stdout.write("Please respond with 'yes' or 'no' "
                             "(or 'y' or 'n').\n")


def response_info(rt):
    request_id = "null"
    code = rt.status_code
    try:
        root = minidom.parseString(rt.content).documentElement
        message = root.getElementsByTagName("Message")[0].childNodes[0].data
        request_id = root.getElementsByTagName(
            "RequestId")[0].childNodes[0].data
    except Exception:
        message = u"Not Found"

    try:
        if request_id == "null":
            request_id = rt.headers['x-cos-request-id']
    except Exception:
        pass
    return (u'''Error: [code {code}] {message}
RequestId: {request_id}'''.format(
        code=code,
        message=message,
        request_id=to_printable_str(request_id)))


def utc_to_local(utc_time_str, utc_format='%Y-%m-%dT%H:%M:%S.%fZ'):
    local_tz = pytz.timezone('Asia/Chongqing')
    local_format = "%Y-%m-%d %H:%M:%S"
    utc_dt = datetime.datetime.strptime(utc_time_str, utc_format)
    local_dt = utc_dt.replace(tzinfo=pytz.utc).astimezone(local_tz)
    time_str = local_dt.strftime(local_format)
    return int(time.mktime(time.strptime(time_str, local_format)))


def change_to_human(_size):
    s = int(_size)
    res = ""
    if s > 1024 * 1024 * 1024:
        res = str(round(1.0 * s / (1024 * 1024 * 1024), 1)) + "G"
    elif s > 1024 * 1024:
        res = str(round(1.0 * s / (1024 * 1024), 1)) + "M"
    elif s > 1024:
        res = str(round(1.0 * s / (1024), 1)) + "K"
    else:
        res = str(s)
    return res



# -*- coding=utf-8
import logging
import random
import string
import sys
import os
import time
import filecmp
import hashlib
from _threading_local import local
logger = logging.getLogger(__name__)
logging.basicConfig(level=logging.INFO, stream=sys.stdout, format="%(asctime)s - %(message)s")
access_id = os.environ["COS_KEY"]
access_key = os.environ["COS_SECRET"]
region = os.environ['COS_REGION']
appid = os.environ['COS_APPID']
bucket_name = "lewzylu" + str(random.randint(0, 1000)) + str(random.randint(0, 1000)) + "-" + appid
special_file_name = "中文" + "→↓←→↖↗↙↘! \"#$%&'()*+,-./0123456789:;<=>@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\\]^_`abcdefghijklmnopqrstuvwxyz{|}~"
file_name = "tmp"
test_file_num = 55
seed = "1234567890abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ!@#$%^&*()_+=-"


def get_raw_md5(data):
    m2 = hashlib.md5(data)
    etag = '"' + str(m2.hexdigest()) + '"'
    return etag


def gen_file(path, size, random_num=2):
    sa = []
    for i in range(8):
        sa.append(random.choice(seed))
    salt = ''.join(sa)
    _file = open(path, 'w')
    for i in range(random_num):
        sk = random.randint(0, int(size * 1024 * 1024))
        _file.seek(sk)
        _file.write(salt)
    _file.close()


def gen_folder(num=1024):
    for i in range(num):
        gen_file("testfolder/testfile中文_" + str(i), 0.01, 2)
    for i in range(12):
        gen_file("testfolder/testfile_big中文_" + str(i), 30, 10)


def setUp():
    """create testbucket"""
    os.system("python setup.py install")
    os.system("python coscmd/cos_cmd.py config -a %s -s %s -b %s -r %s" % (access_id, access_key, bucket_name, region))
    print("创建bucket")
    os.system("python coscmd/cos_cmd.py createbucket >/dev/null 2>&1")
    time.sleep(5)


def tearDown():
    """delete testbucket"""
    print("删除bucket")
    os.system("python coscmd/cos_cmd.py delete -rf / >/dev/null 2>&1")
    os.system("python coscmd/cos_cmd.py deletebucket >/dev/null 2>&1")
    time.sleep(5)


def test_upload_object_1MB():
    """简单上传1MB小文件"""
    gen_file(file_name, 1)

    with open(file_name, 'rb') as f:
        etag = get_raw_md5(f.read())
    rt = os.system("python coscmd/cos_cmd.py upload {local_path} {cos_path} >/dev/null 2>&1".format(local_path=file_name, cos_path=file_name))
    assert rt == 0
    return etag


def test_download_object_1MB():
    """下载1MB小文件"""
    etag = test_upload_object_1MB()
    rt = os.system("python coscmd/cos_cmd.py download -f {cos_path} {local_path} >/dev/null 2>&1".format(local_path=file_name, cos_path=file_name))

    assert rt == 0
    with open(file_name, 'rb') as f:
        etag_download = get_raw_md5(f.read())
    assert etag_download == etag

    if os.path.exists(file_name):
        os.remove(file_name)


def test_upload_object_30MB():
    """简单上传30MB文件"""
    gen_file(file_name, 30)

    with open(file_name, 'rb') as f:
        etag = get_raw_md5(f.read())
    rt = os.system("python coscmd/cos_cmd.py upload {local_path} {cos_path} >/dev/null 2>&1".format(local_path=file_name, cos_path=file_name))
    assert rt == 0
    return etag


def test_download_object_30MB():
    """下载30MB文件"""
    etag = test_upload_object_30MB()
    rt = os.system("python coscmd/cos_cmd.py download -f {cos_path} {local_path} >/dev/null 2>&1".format(local_path=file_name, cos_path=file_name))

    assert rt == 0
    with open(file_name, 'rb') as f:
        etag_download = get_raw_md5(f.read())
    assert etag_download == etag

    if os.path.exists(file_name):
        os.remove(file_name)


def test_delete_object_1MB():
    """删除1MB小文件"""
    test_upload_object_1MB()
    rt = os.system("python coscmd/cos_cmd.py delete -f {cos_path} >/dev/null 2>&1".format(cos_path=file_name))
    assert rt == 0

    if os.path.exists(file_name):
        os.remove(file_name)


def test_bucketacl():
    """Bucket ACL测试"""
    rt = os.system("python coscmd/cos_cmd.py putbucketacl --grant-read anyone --grant-write anyone --grant-full-control anyone >/dev/null 2>&1")
    assert rt == 0
    rt = os.system("python coscmd/cos_cmd.py getbucketacl >/dev/null 2>&1")
    assert rt == 0


def test_probe():
    """探测测试"""
    rt = os.system("python coscmd/cos_cmd.py probe >/dev/null 2>&1")
    assert rt == 0


def test_upload_folder():
    """文件夹上传"""
    try:
        os.makedirs("testfolder/")
    except Exception:
        pass
    gen_folder(test_file_num)

    print("文件夹上传")
    rt = os.system("python coscmd/cos_cmd.py upload -r testfolder testfolder >/dev/null 2>&1")
    assert rt == 0
    print("文件夹同步上传")
    rt = os.system("python coscmd/cos_cmd.py upload -rs testfolder testfolder >/dev/null 2>&1")
    assert rt == 0
    print("文件夹同步上传")
    rt = os.system("python coscmd/cos_cmd.py upload -rs testfolder testfolder --include '*9,*7' >/dev/null 2>&1")
    assert rt == 0
    print("文件夹同步上传")
    rt = os.system("python coscmd/cos_cmd.py upload -rs testfolder testfolder --ignore '*1,*9' >/dev/null 2>&1")
    assert rt == 0
    os.system("rm -rf testfolder/")


def test_download_folder():
    """文件夹下载"""
    try:
        os.makedirs("testfolder/")
    except Exception:
        pass
    gen_folder(test_file_num)
    print("文件夹上传")
    rt = os.system("python coscmd/cos_cmd.py upload -r testfolder testfolder >/dev/null 2>&1")
    assert rt == 0
    time.sleep(5)
    print("文件夹下载")
    rt = os.system("python coscmd/cos_cmd.py download -rf testfolder testfolder >/dev/null 2>&1")
    assert rt == 0
    print("文件夹同步下载")
    rt = os.system("python coscmd/cos_cmd.py download -rsf testfolder testfolder >/dev/null 2>&1")
    print("文件夹同步下载include")
    rt = os.system("python coscmd/cos_cmd.py download -rsf testfolder testfolder --include '*9,*7' >/dev/null 2>&1")
    assert rt == 0
    print("文件夹同步下载ignore")
    rt = os.system("python coscmd/cos_cmd.py download -rsf testfolder testfolder --ignore '*1,*9' >/dev/null 2>&1")
    assert rt == 0
    os.system("rm -rf testfolder/")


def test_copy_folder():
    """文件夹复制"""
    try:
        os.makedirs("testfolder/")
    except Exception:
        pass
    gen_folder(test_file_num)
    print("文件夹上传")
    rt = os.system("python coscmd/cos_cmd.py upload -r testfolder testfolder >/dev/null 2>&1")
    assert rt == 0
    time.sleep(5)
    print("文件夹复制")
    rt = os.system("python coscmd/cos_cmd.py copy -r %s.cos.%s.myqcloud.com/testfolder testfolder2 >/dev/null 2>&1" % (bucket_name, region))
    assert rt == 0
    print("文件夹同步复制include")
    rt = os.system("python coscmd/cos_cmd.py copy -rs %s.cos.%s.myqcloud.com/testfolder testfolder2 --include '*9,*7' >/dev/null 2>&1" % (bucket_name, region))
    assert rt == 0
    print("文件夹同步复制ignore")
    rt = os.system("python coscmd/cos_cmd.py copy -rs %s.cos.%s.myqcloud.com/testfolder testfolder2 --ignore '*1,*9' >/dev/null 2>&1" % (bucket_name, region))
    assert rt == 0
    print("文件夹同步复制--delete")
    rt = os.system("python coscmd/cos_cmd.py copy -rs %s.cos.%s.myqcloud.com/testfolder testfolder2 --ignore '*1,*9' -f --delete >/dev/null 2>&1" % (bucket_name, region))
    assert rt == 0
    print("文件夹move")
    rt = os.system("python coscmd/cos_cmd.py move -r %s.cos.%s.myqcloud.com/testfolder testfolder2 >/dev/null 2>&1" % (bucket_name, region))
    assert rt == 0
    os.system("rm -rf testfolder/")


def test_list_folder():
    """文件夹打印"""
    try:
        os.makedirs("testfolder/")
    except Exception:
        pass
    gen_folder(test_file_num)
    print("文件夹上传")
    rt = os.system("python coscmd/cos_cmd.py upload -r testfolder testfolder >/dev/null 2>&1")
    assert rt == 0
    time.sleep(5)
    print("打印对象")
    rt = os.system("python coscmd/cos_cmd.py list -n 10 >/dev/null 2>&1")
    print("打印全部对象")
    rt = os.system("python coscmd/cos_cmd.py list -ar >/dev/null 2>&1")
    assert rt == 0
    os.system("rm -rf testfolder/")


if __name__ == "__main__":
    setUp()
    tearDown()



# -*- coding: utf-8 -*-
from six.moves.configparser import SafeConfigParser
from six import text_type
from argparse import ArgumentParser
from logging.handlers import RotatingFileHandler
import sys
import logging
import os
import json
import requests
from threading import Thread
from coscmd import cos_global

if sys.version > '3':
    from coscmd.cos_client import CoscmdConfig, CosS3Client
    from coscmd.cos_global import Version
else:
    from cos_client import CoscmdConfig, CosS3Client
    from cos_global import Version

logger = logging.getLogger("coscmd")

fs_coding = sys.getfilesystemencoding()

pre_appid = ""
pre_bucket = ""
config_path = ""
global res


def concat_path(sorce_path, target_path):
    sorce_path = sorce_path.replace('\\', '/')
    target_path = target_path.replace('\\', '/')
    if sorce_path.endswith('/') is False:
        if target_path.endswith('/') is True:
            target_path += sorce_path.split('/')[-1]
    return sorce_path, target_path


def to_printable_str(s):
    if isinstance(s, text_type):
        return s.encode(fs_coding)
    else:
        return s


def config(args):
    logger.debug("config: " + str(args))

    conf_path = os.path.expanduser(config_path)

    with open(conf_path, 'w+') as f:
        cp = SafeConfigParser()
        cp.add_section("common")
        cp.set('common', 'secret_id', args.secret_id)
        cp.set('common', 'secret_key', args.secret_key)
        if args.token != "":
            cp.set('common', 'token', args.token)
        cp.set('common', 'bucket', args.bucket)
        if args.endpoint:
            cp.set('common', 'endpoint', args.endpoint)
        else:
            cp.set('common', 'region', args.region)
        cp.set('common', 'max_thread', str(args.max_thread))
        cp.set('common', 'part_size', str(args.part_size))
        cp.set('common', 'retry', str(args.retry))
        cp.set('common', 'timeout', str(args.timeout))
        if args.appid != "":
            cp.set('common', 'appid', args.appid)
        if args.use_http:
            cp.set('common', 'schema', 'http')
        else:
            cp.set('common', 'schema', 'https')
        cp.set('common', 'verify', args.verify)
        if args.anonymous:
            cp.set('common', 'anonymous', 'True')
        else:
            cp.set('common', 'anonymous', 'False')
        if args.l5:
            cp.set('common', 'l5', args.l5)
        else:
            cp.set('common', 'l5', '')
        cp.write(f)
        logger.info("Created configuration file in {path}".format(path=to_printable_str(conf_path)))


def compatible(region):
    if region is None:
        return None
    _dict = {'tj': 'ap-beijing-1', 'bj': 'ap-beijing', 'gz': 'ap-guangzhou', 'sh': 'ap-shanghai',
             'cd': 'ap-chengdu', 'spg': 'ap-singapore', 'hk': 'ap-hongkong', 'ca': 'na-toronto', 'ger': 'eu-frankfurt',
             'cn-south': 'ap-guangzhou', 'cn-north': 'ap-beijing-1'}
    if region.startswith('cos.'):
        region = region[4:]
    if region in _dict:
        region = _dict[region]
    return region


def load_conf():

    conf_path = os.path.expanduser(config_path)
    if not os.path.exists(conf_path):
        logger.warn("{conf} couldn't be found, please use \'coscmd config -h\' to learn how to config coscmd!".format(conf=to_printable_str(conf_path)))
        raise IOError
    else:
        logger.debug('{conf} is found'.format(conf=to_printable_str(conf_path)))

    try:
        with open(conf_path, 'r') as f:
            cp = SafeConfigParser()
            cp.readfp(fp=f)
            if not cp.has_section('common'):
                raise Exception("[common] section could't be found, please check your config file.")
            if cp.has_option('common', 'part_size'):
                part_size = cp.getint('common', 'part_size')
            else:
                part_size = 1
            if cp.has_option('common', 'max_thread'):
                max_thread = cp.getint('common', 'max_thread')
            else:
                max_thread = 5
            try:
                secret_id = cp.get('common', 'secret_id')
            except Exception:
                secret_id = cp.get('common', 'access_id')
            try:
                appid = cp.get('common', 'appid')
                bucket = cp.get('common', 'bucket')
                if bucket.endswith("-"+str(appid)):
                    bucket = bucket.rstrip(appid)
                    bucket = bucket[:-1]
            except Exception:
                try:
                    bucket = cp.get('common', 'bucket')
                    appid = bucket.split('-')[-1]
                    bucket = bucket.rstrip(appid)
                    bucket = bucket[:-1]
                except Exception:
                    # check if user use -b bucket
                    if (pre_bucket == ""):
                        logger.error("The configuration file is wrong. Please reconfirm")
            try:
                schema = cp.get('common', 'schema')
            except Exception:
                schema = 'https'
            try:
                verify = cp.get('common', 'verify')
            except Exception:
                verify = 'md5'
            try:
                token = cp.get('common', 'token')
            except Exception:
                token = None
            try:
                anonymous = cp.get('common', 'anonymous')
                if anonymous == 'True' or anonymous == 'true':
                    anonymous = True
                else:
                    anonymous = False
            except Exception:
                anonymous = False
            try:
                retry = int(cp.get('common', 'retry'))
            except Exception:
                retry = 2
            try:
                timeout = int(cp.get('common', 'timeout'))
            except Exception:
                timeout = 60
            region, endpoint = None, None
            if cp.has_option('common', 'region'):
                region = cp.get('common', 'region')
            if cp.has_option('common', 'endpoint'):
                endpoint = cp.get('common', 'endpoint')
            if pre_appid != "":
                appid = pre_appid
            if pre_bucket != "":
                bucket = pre_bucket
            if pre_region != "":
                region = pre_region
            try:
                l5 = cp.get('common', 'l5')
            except Exception:
                l5 = ''
            conf = CoscmdConfig(
                appid=appid,
                secret_id=secret_id,
                secret_key=cp.get('common', 'secret_key'),
                token=token,
                region=compatible(region),
                endpoint=endpoint,
                bucket=bucket,
                part_size=part_size,
                max_thread=max_thread,
                schema=schema,
                anonymous=anonymous,
                verify=verify,
                retry=retry,
                timeout=timeout,
                l5=l5
            )
            return conf
    except Exception as e:
        raise(e)


class Op(object):
    @staticmethod
    def upload(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            if args.cos_path == "":
                args.cos_path = "/"
            Interface = client.op_int()

            if not isinstance(args.local_path, text_type):
                args.local_path = args.local_path.decode(fs_coding)
            if not isinstance(args.cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)

            if not os.path.exists(args.local_path):
                logger.warn("cannot stat '%s': No such file or directory" % to_printable_str(args.local_path))
                return -1

            if not os.access(args.local_path, os.R_OK):
                logger.warn('local_path %s is not readable!' % to_printable_str(args.local_path))
                return -1
            args.local_path, args.cos_path = concat_path(args.local_path, args.cos_path)
            if args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            kwargs = {}
            kwargs['sync'] = args.sync
            kwargs['skipmd5'] = args.skipmd5
            kwargs['ignore'] = args.ignore.split(',')
            kwargs['include'] = args.include.split(',')
            kwargs['force'] = args.force
            kwargs['delete'] = args.delete
            kwargs['yes'] = args.yes
            if args.recursive:
                if os.path.isfile(args.local_path) is True:
                    rt = Interface.upload_file(args.local_path, args.cos_path, args.headers, **kwargs)
                    return rt
                elif os.path.isdir(args.local_path):
                    rt = Interface.upload_folder(args.local_path, args.cos_path, args.headers, **kwargs)
                    return rt
            else:
                if os.path.isdir(args.local_path):
                    logger.warn("\"{path}\" is a directory, use \'-r\' option to upload it please".format(path=to_printable_str(args.local_path)))
                    return -1
                if os.path.isfile(args.local_path) is False:
                    logger.warn("cannot stat '%s': No such file or directory" % to_printable_str(args.local_path))
                    return -1
                rt = Interface.upload_file(args.local_path, args.cos_path, args.headers, **kwargs)
                return rt
            return -1
        except Exception as e:
            logger.wran(e)
            return -2

    @staticmethod
    def download(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            if not isinstance(args.local_path, text_type):
                args.local_path = args.local_path.decode(fs_coding)
            if not isinstance(args.cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)
            args.cos_path, args.local_path = concat_path(args.cos_path, args.local_path)
            if args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            kwargs = {}
            kwargs['force'] = args.force
            kwargs['sync'] = args.sync
            kwargs['num'] = min(20, args.num)
            kwargs['ignore'] = args.ignore.split(',')
            kwargs['include'] = args.include.split(',')
            kwargs['skipmd5'] = args.skipmd5
            kwargs['delete'] = args.delete
            kwargs['yes'] = args.yes
            if args.recursive:
                rt = Interface.download_folder(args.cos_path, args.local_path, args.headers, **kwargs)
                return rt
            else:
                rt = Interface.download_file(args.cos_path, args.local_path, args.headers, **kwargs)
                return rt
            return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def delete(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            Interface = client.op_int()

            if not isinstance(args. cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)

            kwargs = {}
            kwargs['force'] = args.force
            kwargs['versions'] = args.versions
            kwargs['versionId'] = args.versionId
            kwargs['yes'] = args.yes
            if args.recursive:
                if args.cos_path.endswith('/') is False:
                    args.cos_path += '/'
                if args.cos_path == '/':
                    args.cos_path = ''
                if not Interface.delete_folder(args.cos_path, **kwargs):
                    logger.debug("delete all files under {cos_path} successfully!".format(cos_path=to_printable_str(args.cos_path)))
                    return 0
                else:
                    logger.debug("delete all files under {cos_path} failed!".format(cos_path=to_printable_str(args.cos_path)))
                    return -1
            else:
                if args.cos_path == '':
                    logger.warn("not support delete empty path")
                    return -1
                if not Interface.delete_file(args.cos_path, **kwargs):
                    logger.debug("delete all files under {cos_path} successfully!".format(cos_path=to_printable_str(args.cos_path)))
                    return 0
                else:
                    logger.debug("delete all files under {cos_path} failed!".format(cos_path=to_printable_str(args.cos_path)))
                    return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def copy(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            _, args.cos_path = concat_path(args.source_path, args.cos_path)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            if not isinstance(args.source_path, text_type):
                args.source_path = args.source_path.decode(fs_coding)
            if not isinstance(args.cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)

            kwargs = {}
            kwargs['sync'] = args.sync
            kwargs['force'] = args.force
            kwargs['directive'] = args.directive
            kwargs['skipmd5'] = args.skipmd5
            kwargs['ignore'] = args.ignore.split(',')
            kwargs['include'] = args.include.split(',')
            kwargs['delete'] = args.delete
            kwargs['yes'] = args.yes
            kwargs['move'] = False
            if args.recursive:
                _, args.cos_path = concat_path(args.source_path, args.cos_path)
                if args.cos_path.endswith('/') is False:
                    args.cos_path += '/'
                if args.cos_path.startswith('/'):
                    args.cos_path = args.cos_path[1:]
                if not Interface.copy_folder(args.source_path, args.cos_path, args.headers, **kwargs):
                    return 0
                else:
                    return 1
            else:
                if not Interface.copy_file(args.source_path, args.cos_path, args.headers, **kwargs):
                    return 0
                else:
                    return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def move(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            _, args.cos_path = concat_path(args.source_path, args.cos_path)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            if not isinstance(args.source_path, text_type):
                args.source_path = args.source_path.decode(fs_coding)
            if not isinstance(args.cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)

            kwargs = {}
            kwargs['sync'] = False
            kwargs['force'] = True
            kwargs['directive'] = args.directive
            kwargs['skipmd5'] = True
            kwargs['ignore'] = args.ignore.split(',')
            kwargs['include'] = args.include.split(',')
            kwargs['delete'] = False
            kwargs['move'] = True
            if args.recursive:
                _, args.cos_path = concat_path(args.source_path, args.cos_path)
                if args.cos_path.endswith('/') is False:
                    args.cos_path += '/'
                if args.cos_path.startswith('/'):
                    args.cos_path = args.cos_path[1:]
                if not Interface.copy_folder(args.source_path, args.cos_path, args.headers, **kwargs):
                    return 0
                else:
                    return 1
            else:
                if not Interface.copy_file(args.source_path, args.cos_path, args.headers, **kwargs):
                    return 0
                else:
                    return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def list(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            if not isinstance(args. cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)
            Interface = client.op_int()
            kwargs = {}
            kwargs['recursive'] = args.recursive
            kwargs['all'] = args.all
            kwargs['num'] = args.num
            kwargs['human'] = args.human
            kwargs['versions'] = args.versions
            if not Interface.list_objects(cos_path=args.cos_path, **kwargs):
                return 0
            else:
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def list_parts(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            if Interface.list_multipart_uploads(cos_path=args.cos_path):
                return 0
            else:
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def abort(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]

            if not isinstance(args. cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)
            Interface = client.op_int()
            if not Interface.abort_parts(cos_path=args.cos_path):
                return 0
            else:
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def info(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]

            if not isinstance(args. cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)
            Interface = client.op_int()
            if not Interface.info_object(args.cos_path, _human=args.human):
                return 0
            else:
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def restore(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]

            if not isinstance(args. cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)
            Interface = client.op_int()
            kwargs = {}
            kwargs['day'] = args.day
            kwargs['tier'] = args.tier
            if args.recursive:
                if not Interface.restore_folder(cos_path=args.cos_path, **kwargs):
                    return 0
                else:
                    return -1
            else:
                if not Interface.restore_file(cos_path=args.cos_path, **kwargs):
                    return 0
                else:
                    return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def signurl(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            if not isinstance(args.cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            try:
                Interface = client.op_int()
                rt = Interface.sign_url(args.cos_path, args.timeout)
                if rt:
                    return 0
                else:
                    return -1
            except Exception:
                logger.warn('Geturl fail')
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def put_object_acl(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            if not isinstance(args. cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)
            Interface = client.op_int()
            rt = Interface.put_object_acl(args.grant_read, args.grant_write, args.grant_full_control, args.cos_path)
            if rt is True:
                return 0
            else:
                logger.warn("Put object acl fail")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def get_object_acl(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            while args.cos_path.startswith('/'):
                args.cos_path = args.cos_path[1:]
            if not isinstance(args. cos_path, text_type):
                args.cos_path = args.cos_path.decode(fs_coding)
            Interface = client.op_int()

            rt = Interface.get_object_acl(args.cos_path)
            if rt is True:
                return 0
            else:
                logger.warn("Get object acl fail")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def create_bucket(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            if Interface.create_bucket():
                return 0
            else:
                logger.warn("Create bucket fail")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def delete_bucket(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            kwargs = {}
            kwargs['force'] = args.force
            if Interface.delete_bucket(**kwargs):
                return 0
            else:
                logger.warn("Delete bucket fail")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def put_bucket_acl(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            rt = Interface.put_bucket_acl(args.grant_read, args.grant_write, args.grant_full_control)
            if rt is True:
                return 0
            else:
                logger.warn("put bucket acl fail")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def get_bucket_acl(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            rt = Interface.get_bucket_acl()
            if rt is True:
                return 0
            else:
                logger.warn("Get bucket acl fail")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def put_bucket_versioning(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            rt = Interface.put_bucket_versioning(args.status)
            if rt is True:
                return 0
            else:
                logger.warn("Put bucket versioning fail")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def get_bucket_versioning(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            rt = Interface.get_bucket_versioning()
            if rt is True:
                return 0
            else:
                logger.warn("Get bucket versioning fail")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2

    @staticmethod
    def probe(args):
        try:
            conf = load_conf()
            client = CosS3Client(conf)
            Interface = client.op_int()
            kwargs = {}
            kwargs['test_num'] = args.num
            kwargs['file_size'] = args.size
            rt = Interface.probe(**kwargs)
            if 0 == rt:
                return 0
            else:
                logger.warn("probe failed")
                return -1
        except Exception as e:
            logger.warn(e)
            return -2


def get_version():
    logger.info(Version)
    return 0


def version_check():
    try:
        ret = requests.get("https://pypi.org/pypi/coscmd/json").content
        res_json = json.loads(ret)
        latest_version = res_json["info"]["version"]
        lat_spl = latest_version.split('.')
        cur_spl = cos_global.Version.split('.')
        if cur_spl[0] < lat_spl[0] or cur_spl[1] < lat_spl[1] or cur_spl[2] < lat_spl[2]:
            logger.info("The current version of coscmd is {v1} \
and the latest version is {v2}. It is recommended \
to upgrade coscmd with the command'pip install coscmd -U'.".format(v1=cos_global.Version, v2=latest_version))
    except Exception as e:
        logger.debug(e)


def command_thread():
    global res
    res = -1
    desc = """an easy-to-use but powerful command-line tool.
              try \'coscmd -h\' to get more informations.
              try \'coscmd sub-command -h\' to learn all command usage, likes \'coscmd upload -h\'"""
    parser = ArgumentParser(description=desc)
    parser.add_argument('-d', '--debug', help="Debug mode", action="store_true", default=False)
    parser.add_argument('-b', '--bucket', help="Specify bucket", type=str, default="")
    parser.add_argument('-r', '--region', help="Specify region", type=str, default="")
    parser.add_argument('-c', '--config_path', help="Specify config_path", type=str, default="~/.cos.conf")
    parser.add_argument('-l', '--log_path', help="Specify log_path", type=str, default="~/.cos.log")
    parser.add_argument('--log_size', help='specify max log size in MB (default 1MB)', type=int, default=128)
    parser.add_argument('--log_backup_count', help='specify log backup num', type=int, default=1)

    sub_parser = parser.add_subparsers()
    parser_config = sub_parser.add_parser("config", help="Config your information at first")
    parser_config.add_argument('-a', '--secret_id', help='Specify your secret id', type=str, required=True)
    parser_config.add_argument('-s', '--secret_key', help='Specify your secret key', type=str, required=True)
    parser_config.add_argument('-t', '--token', help='Set x-cos-security-token header', type=str, default="")
    parser_config.add_argument('-b', '--bucket', help='Specify your bucket', type=str, required=True)

    group = parser_config.add_mutually_exclusive_group(required=True)
    group.add_argument('-r', '--region', help='Specify your region', type=str)
    group.add_argument('-e', '--endpoint', help='Specify COS endpoint', type=str)

    parser_config.add_argument('-m', '--max_thread', help='Specify the number of threads (default 5)', type=int, default=5)
    parser_config.add_argument('-p', '--part_size', help='specify min part size in MB (default 1MB)', type=int, default=1)
    parser_config.add_argument('--retry', help='specify retry times', type=int, default=5)
    parser_config.add_argument('--timeout', help='specify request timeout', type=int, default=60)
    parser_config.add_argument('-u', '--appid', help='Specify your appid', type=str, default="")
    parser_config.add_argument('--verify', help='Specify your encryption method', type=str, default="md5")
    parser_config.add_argument('--do-not-use-ssl', help="Use http://", action="store_true", default=False, dest="use_http")
    parser_config.add_argument('--anonymous', help="Anonymous operation", action="store_true", default=False, dest="anonymous")
    parser_config.add_argument('--l5', help="l5", type=str, default='', dest="l5")
    parser_config.set_defaults(func=config)

    parser_upload = sub_parser.add_parser("upload", help="Upload file or directory to COS")
    parser_upload.add_argument('local_path', help="Local file path as /tmp/a.txt or directory", type=str)
    parser_upload.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_upload.add_argument('-r', '--recursive', help="Upload recursively when upload directory", action="store_true", default=False)
    parser_upload.add_argument('-H', '--headers', help="Specify HTTP headers", type=str, default='{}')
    parser_upload.add_argument('-s', '--sync', help="Upload and skip the same file", action="store_true", default=False)
    parser_upload.add_argument('-f', '--force', help="upload without history breakpoint", action="store_true", default=False)
    parser_upload.add_argument('-y', '--yes', help="Skip confirmation", action="store_true", default=False)
    parser_upload.add_argument('--include', help='Specify filter rules, separated by commas; Example: *.txt,*.docx,*.ppt', type=str, default="*")
    parser_upload.add_argument('--ignore', help='Specify ignored rules, separated by commas; Example: *.txt,*.docx,*.ppt', type=str, default="")
    parser_upload.add_argument('--skipmd5', help='Upload without x-cos-meta-md5 / sync without check md5, only check filename and filesize', action="store_true", default=False)
    parser_upload.add_argument('--delete', help="delete objects which exists in cos but not exist in local", action="store_true", default=False)
    parser_upload.set_defaults(func=Op.upload)

    parser_download = sub_parser.add_parser("download", help="Download file from COS to local")
    parser_download.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_download.add_argument('local_path', help="Local file path as /tmp/a.txt", type=str)
    parser_download.add_argument('-f', '--force', help="Overwrite the saved files", action="store_true", default=False)
    parser_download.add_argument('-y', '--yes', help="Skip confirmation", action="store_true", default=False)
    parser_download.add_argument('-r', '--recursive', help="Download recursively when upload directory", action="store_true", default=False)
    parser_download.add_argument('-s', '--sync', help="Download and skip the same file", action="store_true", default=False)
    parser_download.add_argument('-H', '--headers', help="Specify HTTP headers", type=str, default='{}')
    parser_download.add_argument('--versionId', help='Specify versionId of object to list', type=str, default="")
    parser_download.add_argument('--include', help='Specify filter rules, separated by commas; Example: *.txt,*.docx,*.ppt', type=str, default="*")
    parser_download.add_argument('--ignore', help='Specify ignored rules, separated by commas; Example: *.txt,*.docx,*.ppt', type=str, default="")
    parser_download.add_argument('--skipmd5', help='Download sync without check md5, only check filename and filesize', action="store_true", default=False)
    parser_download.add_argument('--delete', help="delete objects which exists in local but not exist in cos", action="store_true", default=False)
    parser_download.add_argument('-n', '--num', help='Specify max part_num of multidownload', type=int, default=10)
    parser_download.set_defaults(func=Op.download)

    parser_delete = sub_parser.add_parser("delete", help="Delete file or files on COS")
    parser_delete.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_delete.add_argument('-r', '--recursive', help="Delete files recursively, WARN: all files with the prefix will be deleted!", action="store_true", default=False)
    parser_delete.add_argument('--versions', help='Delete objects with versions', action="store_true", default=False)
    parser_delete.add_argument('--versionId', help='Specify versionId of object to list', type=str, default="")
    parser_delete.add_argument('-f', '--force', help="Delete directly without confirmation", action="store_true", default=False)
    parser_delete.add_argument('-y', '--yes', help="Delete directly without confirmation", action="store_true", default=False)
    parser_delete.set_defaults(func=Op.delete)

    parser_abort = sub_parser.add_parser("abort", help='Aborts upload parts on COS')
    parser_abort.add_argument("cos_path", nargs='?', help="Cos_path as a/b.txt", type=str, default='')
    parser_abort.set_defaults(func=Op.abort)

    parser_copy = sub_parser.add_parser("copy", help="Copy file from COS to COS")
    parser_copy.add_argument('source_path', help="Source file path as 'bucket-appid.cos.ap-guangzhou.myqcloud.com/a.txt'", type=str)
    parser_copy.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_copy.add_argument('-H', '--headers', help="Specify HTTP headers", type=str, default='{}')
    parser_copy.add_argument('-d', '--directive', help="if Overwrite headers", type=str, choices=['Copy', 'Replaced'], default="Copy")
    parser_copy.add_argument('-s', '--sync', help="Copy and skip the same file", action="store_true", default=False)
    parser_copy.add_argument('-r', '--recursive', help="Copy files recursively", action="store_true", default=False)
    parser_copy.add_argument('-f', '--force', help="Overwrite file without skip", action="store_true", default=False)
    parser_copy.add_argument('-y', '--yes', help="Skip confirmation", action="store_true", default=False)
    parser_copy.add_argument('--include', help='Specify filter rules, separated by commas; Example: *.txt,*.docx,*.ppt', type=str, default="*")
    parser_copy.add_argument('--ignore', help='Specify ignored rules, separated by commas; Example: *.txt,*.docx,*.ppt', type=str, default="")
    parser_copy.add_argument('--skipmd5', help='Copy sync without check md5, only check filename and filesize', action="store_true", default=False)
    parser_copy.add_argument('--delete', help="delete objects which exists in sourcepath but not exist in dstpath", action="store_true", default=False)
    parser_copy.set_defaults(func=Op.copy)

    parser_move = sub_parser.add_parser("move", help="move file from COS to COS")
    parser_move.add_argument('source_path', help="Source file path as 'bucket-appid.cos.ap-guangzhou.myqcloud.com/a.txt'", type=str)
    parser_move.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_move.add_argument('-H', '--headers', help="Specify HTTP headers", type=str, default='{}')
    parser_move.add_argument('-d', '--directive', help="if Overwrite headers", type=str, choices=['Copy', 'Replaced'], default="Copy")
    parser_move.add_argument('-r', '--recursive', help="Copy files recursively", action="store_true", default=False)
    parser_move.add_argument('--include', help='Specify filter rules, separated by commas; Example: *.txt,*.docx,*.ppt', type=str, default="*")
    parser_move.add_argument('--ignore', help='Specify ignored rules, separated by commas; Example: *.txt,*.docx,*.ppt', type=str, default="")
    parser_move.set_defaults(func=Op.move)

    parser_list = sub_parser.add_parser("list", help='List files on COS')
    parser_list.add_argument("cos_path", nargs='?', help="Cos_path as a/b.txt", type=str, default='')
    parser_list.add_argument('-a', '--all', help="List all the files", action="store_true", default=False)
    parser_list.add_argument('-r', '--recursive', help="List files recursively", action="store_true", default=False)
    parser_list.add_argument('-n', '--num', help='Specify max num of files to list', type=int, default=100)
    parser_list.add_argument('-v', '--versions', help='List object with versions', action="store_true", default=False)
    parser_list.add_argument('--human', help='Humanized display', action="store_true", default=False)
    parser_list.set_defaults(func=Op.list)

    parser_list_parts = sub_parser.add_parser("listparts", help="List upload parts")
    parser_list_parts.add_argument("cos_path", nargs='?', help="Cos_path as a/b.txt", type=str, default='')
    parser_list_parts.set_defaults(func=Op.list_parts)

    parser_info = sub_parser.add_parser("info", help="Get the information of file on COS")
    parser_info.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_info.add_argument('--human', help='Humanized display', action="store_true", default=False)
    parser_info.set_defaults(func=Op.info)

    parser_restore = sub_parser.add_parser("restore", help="Restore")
    parser_restore.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_restore.add_argument('-r', '--recursive', help="Restore files recursively", action="store_true", default=False)
    parser_restore.add_argument('-d', '--day', help='Specify lifetime of the restored (active) copy', type=int, default=7)
    parser_restore.add_argument('-t', '--tier', help='Specify the data access tier', type=str, choices=['Expedited', 'Standard', 'Bulk'], default='Standard')
    parser_restore.set_defaults(func=Op.restore)

    parser_signurl = sub_parser.add_parser("signurl", help="Get download url")
    parser_signurl.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_signurl.add_argument('-t', '--timeout', help='Specify the signature valid time', type=int, default=10000)
    parser_signurl.set_defaults(func=Op.signurl)

    parser_create_bucket = sub_parser.add_parser("createbucket", help='Create bucket')
    parser_create_bucket.set_defaults(func=Op.create_bucket)

    parser_delete_bucket = sub_parser.add_parser("deletebucket", help='Delete bucket')
    parser_delete_bucket.add_argument('-f', '--force', help="Clear all inside the bucket and delete bucket", action="store_true", default=False)
    parser_delete_bucket.set_defaults(func=Op.delete_bucket)

    parser_put_object_acl = sub_parser.add_parser("putobjectacl", help='''Set object acl''')
    parser_put_object_acl.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_put_object_acl.add_argument('--grant-read', dest='grant_read', help='Set grant-read', type=str, required=False)
    parser_put_object_acl.add_argument('--grant-write', dest='grant_write', help='Set grant-write', type=str, required=False)
    parser_put_object_acl.add_argument('--grant-full-control', dest='grant_full_control', help='Set grant-full-control', type=str, required=False)
    parser_put_object_acl.set_defaults(func=Op.put_object_acl)

    parser_get_object_acl = sub_parser.add_parser("getobjectacl", help='Get object acl')
    parser_get_object_acl.add_argument("cos_path", help="Cos_path as a/b.txt", type=str)
    parser_get_object_acl.set_defaults(func=Op.get_object_acl)

    parser_put_bucket_acl = sub_parser.add_parser("putbucketacl", help='''Set bucket acl''')
    parser_put_bucket_acl.add_argument('--grant-read', dest='grant_read', help='Set grant-read', type=str, required=False)
    parser_put_bucket_acl.add_argument('--grant-write', dest='grant_write', help='Set grant-write', type=str, required=False)
    parser_put_bucket_acl.add_argument('--grant-full-control', dest='grant_full_control', help='Set grant-full-control', type=str, required=False)
    parser_put_bucket_acl.set_defaults(func=Op.put_bucket_acl)

    parser_get_bucket_acl = sub_parser.add_parser("getbucketacl", help='Get bucket acl')
    parser_get_bucket_acl.set_defaults(func=Op.get_bucket_acl)

    parser_put_bucket_versioning = sub_parser.add_parser("putbucketversioning", help="Set the versioning state")
    parser_put_bucket_versioning.add_argument("status",  help="Status as a/b.txt", type=str, choices=['Enabled', 'Suspended'], default='Enable')
    parser_put_bucket_versioning.set_defaults(func=Op.put_bucket_versioning)

    parser_get_bucket_versioning = sub_parser.add_parser("getbucketversioning", help="Get the versioning state")
    parser_get_bucket_versioning.set_defaults(func=Op.get_bucket_versioning)

    parser_probe = sub_parser.add_parser("probe", help="Connection test")
    parser_probe.add_argument('-n', '--num', help='Specify test times', type=int, default=3)
    parser_probe.add_argument('-s', '--size', help='Specify test filesize(unit MB)', type=int, default=1)
    parser_probe.set_defaults(func=Op.probe)

    parser.add_argument('-v', '--version', action='version', version='%(prog)s ' + Version)

    args = parser.parse_args()

    logger = logging.getLogger('coscmd')
    logger.setLevel(logging.INFO)
    console = logging.StreamHandler()
    console.setLevel(logging.INFO)
    if args.debug:
        logger.setLevel(logging.DEBUG)
        console.setLevel(logging.DEBUG)
    handler = RotatingFileHandler(os.path.expanduser(args.log_path), maxBytes=args.log_size*1024*1024, backupCount=args.log_backup_count)
    handler.setFormatter(logging.Formatter('%(asctime)s - [%(levelname)s]:  %(message)s'))
    logger.addHandler(handler)
    logging.getLogger('coscmd').addHandler(console)
    global pre_appid, pre_bucket, pre_region, config_path
    config_path = args.config_path
    pre_bucket = args.bucket
    pre_region = args.region
    try:
        pre_appid = pre_bucket.split('-')[-1]
        pre_bucket = pre_bucket.rstrip(pre_appid)
        pre_bucket = pre_bucket[:-1]
    except Exception:
        logger.warn("set bucket error")
    try:
        res = args.func(args)
        return res
    except Exception:
        return 0


def main_thread():
    mainthread = Thread()
    mainthread.start()
    thread_ = Thread(target=command_thread)
    thread_.start()
    import time
    try:
        while True:
            time.sleep(1)
            if thread_.is_alive() is False:
                break
    except KeyboardInterrupt:
        mainthread.stop()
        thread_.stop()
        sys.exit()


def _main():
    thread_ = Thread(target=main_thread)
    thread_.daemon = True
    thread_.start()
    try:
        while thread_.is_alive():
            thread_.join(2)
    except KeyboardInterrupt:
        logger.info('exiting')
        return 1
    global res
    return res


if __name__ == '__main__':
    _main()
    global res
    sys.exit(res)



# -*- coding: utf-8 -*-
import fnmatch
import os
import sys
from six.moves.queue import Queue
from qcloud_cos import CosServiceError
if sys.version > '3':
    from coscmd.cos_comm import *
else:
    from cos_comm import *

logger = logging.getLogger("coscmd")


def is_include_file(path, rules):
    for rule in rules:
        if fnmatch.fnmatch(path, rule) is True:
            return True


def is_ignore_file(path, rules):
    for rule in rules:
        if fnmatch.fnmatch(path, rule) is True:
            return True


def is_sync_skip_file_remote2local(cos_path, local_path, **kwargs):
    """
    校验是否进行下载sync
    """
    if os.path.isfile(local_path) is False:
        return False
    if not kwargs['skipmd5'] and "_md5" in kwargs:
        _md5 = get_file_md5(local_path)
        if _md5 != kwargs["_md5"]:
            return False
    if "_size" in kwargs:
        _size = os.path.getsize(local_path)
        if _size != kwargs["_size"]:
            return False
    else:
        return False
    return True


def delete_objects(src, deleteList):
    """
    批量删除cos上的对象
    """
    success_num = 0
    fail_num = 0
    rt = {}
    try:
        if len(deleteList['Object']) > 0:
            rt = src['Client'].delete_objects(Bucket=src['Bucket'],
                                              Delete=deleteList)
        if 'Deleted' in rt:
            success_num += len(rt['Deleted'])
            for file in rt['Deleted']:
                logger.info(u"Delete cos://{bucket}/{file}".format(bucket=src['Bucket'],
                                                                   file=file['Key'],))
        if 'Error' in rt:
            fail_num += len(rt['Error'])
            for file in rt['Error']:
                logger.info(u"Delete cos://{bucket}/{file} fail, code: {code}, msg: {msg}"
                            .format(bucket=src['Bucket'],
                                    file=file['Key'],
                                    code=file['Code'],
                                    msg=file['Message']))
    except Exception as e:
        logger.warn(e)
        return [0, len(deleteList['Object'])]
    return [success_num, fail_num]


def local2remote_sync_delete(src, dst, **kwargs):
    """
    上传sync时携带--delete，删除cos上存在而本地不存在的对象
    """
    success_num = 0
    fail_num = 0
    NextMarker = ""
    IsTruncated = "true"
    rt = {}
    while IsTruncated == "true":
        deleteList = {}
        deleteList['Object'] = []
        for i in range(kwargs['retry']):
            try:
                rt = dst['Client'].list_objects(
                    Bucket=dst['Bucket'],
                    Marker=NextMarker,
                    MaxKeys=1000,
                    Delimiter="",
                    Prefix=dst['Path'],
                )
                if 'IsTruncated' in rt:
                    IsTruncated = rt['IsTruncated']
                if 'NextMarker' in rt:
                    NextMarker = rt['NextMarker']
                if 'Contents' in rt:
                    for _file in rt['Contents']:
                        _cos_path = to_unicode(_file['Key'])
                        _local_path = src['Path'] + _cos_path[len(dst['Path']):]
                        _local_path = to_unicode(_local_path)
                        if os.path.isfile(_local_path) is False:
                            deleteList['Object'].append({'Key': _cos_path})
                break
            except Exception as e:
                time.sleep(1 << i)
                logger.warn(e)
            if i + 1 == kwargs['retry']:
                return [-1, 0, 0]
        _succ, _fail = delete_objects(dst, deleteList)
        success_num += _succ
        fail_num += _fail
    return [0, success_num, fail_num]


def remote2local_sync_delete(src, dst, **kwargs):
    """
    下载sync时携带--delete，删除本地存在而cos上不存在的对象
    """
    q = Queue()
    q.put([dst['Path'], src['Path']])
    success_num = 0
    fail_num = 0
    # BFS上传文件夹
    try:
        while(not q.empty()):
            [local_path, cos_path] = q.get()
            local_path = to_unicode(local_path)
            cos_path = to_unicode(cos_path)
            if cos_path.endswith('/') is False:
                cos_path += "/"
            if local_path.endswith('/') is False:
                local_path += "/"
            cos_path = cos_path.lstrip('/')
            # 当前目录下的文件列表
            dirlist = os.listdir(local_path)
            for filename in dirlist:
                filepath = os.path.join(local_path, filename)
                if os.path.isdir(filepath):
                    q.put([filepath, cos_path + filename])
                else:
                    try:
                        src['Client'].head_object(Bucket=src['Bucket'],
                                                  Key=cos_path + filename)
                    except CosServiceError as e:
                        if e.get_status_code() == 404:
                            try:
                                os.remove(filepath)
                                logger.info(u"Delete {file}".format(
                                    file=filepath))
                                success_num += 1
                            except Exception:
                                logger.info(u"Delete {file} fail".format(
                                    file=filepath))
                                fail_num += 1
    except Exception as e:
        logger.warn(e)
        return [-1, 0, 0]
    return [0, success_num, fail_num]


def remote2remote_sync_delete(src, dst, **kwargs):
    """
    复制sync时携带--delete，删除cos上存在而本地不存在的对象
    """
    success_num = 0
    fail_num = 0
    NextMarker = ""
    IsTruncated = "true"
    rt = {}
    while IsTruncated == "true":
        deleteList = {}
        deleteList['Object'] = []
        for i in range(kwargs['retry']):
            try:
                rt = dst['Client'].list_objects(
                    Bucket=dst['Bucket'],
                    Marker=NextMarker,
                    MaxKeys=1000,
                    Delimiter="",
                    Prefix=dst['Path'],
                )
                if 'IsTruncated' in rt:
                    IsTruncated = rt['IsTruncated']
                if 'NextMarker' in rt:
                    NextMarker = rt['NextMarker']
                if 'Contents' in rt:
                    for _file in rt['Contents']:
                        _cos_path = to_unicode(_file['Key'])
                        _source_path = src['Path'] + _cos_path[len(dst['Path']):]
                        _source_path = to_unicode(_source_path)
                        try:
                            src['Client'].head_object(
                                Bucket=src['Bucket'],
                                Key=_source_path,
                            )
                        except CosServiceError as e:
                            if e.get_status_code() == 404:
                                deleteList['Object'].append({'Key': _cos_path})
                break
            except Exception as e:
                time.sleep(1 << i)
                logger.warn(e)
            if i + 1 == kwargs['retry']:
                return [-1, 0, 0]
        _succ, _fail = delete_objects(dst, deleteList)
        success_num += _succ
        fail_num += _fail
    return [0, success_num, fail_num]



# -*- coding: utf-8 -*-
import hmac
import time
import urllib
import hashlib
import logging
import requests
import sys
from six.moves.urllib.parse import quote, urlparse, unquote, urlencode
from requests.auth import AuthBase

if sys.version > '3':
    from coscmd.cos_global import Version
    from coscmd.cos_comm import to_bytes
else:
    from cos_global import Version
    from cos_comm import to_bytes

logger = logging.getLogger("coscmd")


class CosS3Auth(AuthBase):

    def __init__(self, conf, expire=10000):
        self._access_id = conf._secret_id
        self._secret_key = conf._secret_key
        self._anonymous = conf._anonymous
        self._token = conf._token
        self._expire = expire
        self._oldurl = conf._oldurl

    def __call__(self, r):
        method = r.method.lower()
        uri = r.url
        if self._oldurl != '':
            uri = self._oldurl
        uri = uri.split('?')[0]
        tmp_r = {}
        rt = urlparse(uri)
        if rt.query != "" and ("&" in rt.query or '=' in rt.query):
            uri_params = dict(map(lambda s: s.lower().split('='), rt.query.split('&')))
        elif rt.query != "":
            uri_params = {rt.query: ""}
        else:
            uri_params = {}
        tmp_r = {}
        tmp_r = r.headers
        r.headers = {}
        r.headers['Host'] = rt.netloc
        headers = dict([(k.lower(), quote(v).lower()) for k, v in r.headers.items()])
        format_str = "{method}\n{host}\n{params}\n{headers}\n".format(
            method=method.lower(),
            host=unquote(rt.path),
            params=urlencode(uri_params),
            headers='&'.join(map(lambda p: (lambda x, y: "%s=%s" % (x, y))(*p), sorted(headers.items())))
         )
        start_sign_time = int(time.time())
        sign_time = "{bg_time};{ed_time}".format(bg_time=start_sign_time-60, ed_time=start_sign_time + self._expire)
        sha1 = hashlib.sha1()
        sha1.update(to_bytes(format_str))

        str_to_sign = "sha1\n{time}\n{sha1}\n".format(time=sign_time, sha1=sha1.hexdigest())

        sign_key = hmac.new(to_bytes(self._secret_key), to_bytes(sign_time), hashlib.sha1).hexdigest()
        sign = hmac.new(to_bytes(sign_key), to_bytes(str_to_sign), hashlib.sha1).hexdigest()
        sign_tpl = "q-sign-algorithm=sha1&q-ak={ak}&q-sign-time={sign_time}&q-key-time={key_time}&q-header-list={headers}&q-url-param-list={params}&q-signature={sign}"
        r.headers = tmp_r
        r.headers['Authorization'] = sign_tpl.format(
            ak=self._access_id,
            sign_time=sign_time,
            key_time=sign_time,
            params=';'.join(sorted(map(lambda k: k.lower(), uri_params.keys()))),
            headers=';'.join(sorted(headers.keys())),
            sign=sign
        )
        if self._anonymous:
            r.headers['Authorization'] = ""
        if self._token is not None:
            r.headers['x-cos-security-token'] = self._token
        r.headers['User-agent'] = 'coscmd-v' + Version
        return r


if __name__ == "__main__":
    url = 'http://Lewzylu01-1252448703.cn-south.myqcloud.com/a.txt'
    request = requests.session()
    secret_id = 'AKID15IsskiBQKTZbAo6WhgcBqVls9SmuG00'
    secret_key = 'ciivKvnnrMvSvQpMAWuIz12pThGGlWRW'
    rt = request.get(url=url+"", auth=CosS3Auth(secret_id, secret_key))



# -*- coding: utf-8 -*-

from threading import Thread
from logging import getLogger
from six.moves.queue import Queue
from threading import Lock
import gc
import time
logger = getLogger("coscmd")


class WorkerThread(Thread):
    def __init__(self, task_queue, *args, **kwargs):
        super(WorkerThread, self).__init__(*args, **kwargs)

        self._task_queue = task_queue
        self._succ_task_num = 0
        self._fail_task_num = 0
        self._ret = list()

    def run(self):
        while True:
            func, args, kwargs = self._task_queue.get()

            try:
                ret = func(*args, **kwargs)
                self._succ_task_num += 1
                self._ret.append(ret)

            except Exception as e:
                logger.warn(str(e))
                self._fail_task_num += 1
                self._ret.append(e)
            finally:
                self._task_queue.task_done()
            if self._task_queue.empty():
                break

    def get_result(self):
        return self._succ_task_num, self._fail_task_num, self._ret


class SimpleThreadPool:

    def __init__(self, num_threads=5):
        self._num_threads = num_threads
        self._queue = Queue(2000)
        self._lock = Lock()
        self._active = False
        self._workers = list()
        self._finished = False

    def add_task(self, func, *args, **kwargs):
        if not self._active:
            with self._lock:
                if not self._active:
                    self._active = True
                    for i in range(self._num_threads):
                        w = WorkerThread(self._queue)
                        self._workers.append(w)
                        w.start()

        self._queue.put((func, args, kwargs))

    def release(self):
        while self._queue.empty() is False:
            time.sleep(1)

    def wait_completion(self):
        self._queue.join()
        self._finished = True

    def complete(self):
        self._finished = True

    def get_result(self):
        assert self._finished
        detail = [worker.get_result() for worker in self._workers]
        succ_all = all([tp[1] == 0 for tp in detail])
        return {'success_all': succ_all, 'detail': detail}


if __name__ == '__main__':

    pool = SimpleThreadPool(2)

    def task_sleep(x):
        from time import sleep
        sleep(x)
        return 'hello, sleep %d seconds' % x

    def raise_exception():
        raise ValueError("Pa! Exception!")
    for i in range(1000):
        pool.add_task(task_sleep, 0.001)
        print(i)
    pool.add_task(task_sleep, 0)
    pool.add_task(task_sleep, 0)
    # pool.add_task(raise_exception)
    # pool.add_task(raise_exception)

    pool.wait_completion()
    print(pool.get_result())
    # [(1, 0, ['hello, sleep 5 seconds']), (2, 1, ['hello, sleep 2 seconds', 'hello, sleep 3 seconds', ValueError('Pa! Exception!',)])]
